# Installing the required packages for this analysis. 
# You will only need to do this once.
# Uncomment the code below to run the install.packages function.
# install.packages(c("eRm", "dplyr", "ggplot2", "pairwise", "TAM", "psych"))

# Load the packages required for the analysis
library(eRm)
library(dplyr)
library(ggplot2)

# Reading in the comma-seperated data set
df <- read.csv("ARMChapter5_PRTIII.csv", header = FALSE, sep = ";") 

# Defining the names of the columns
colnames(df)[2:14] <- paste("PRTIII", 1:13, sep="_") 
# Selecting only the PRTIII items
prt_items <- dplyr::select(df, PRTIII_1:PRTIII_13) 
# Looking at the first five rows of the PRTIII dataframe
head(prt_items)

# Reading in the comma-seperated data set for the BLOT data
df2 <- read.csv("ARMChapter5.csv", header = FALSE, sep = ";") 
# Defining the name of the first column of the data frame called "df2"
colnames(df2)[1] <- c("group") 
colnames(df2)[2] <- c("gender") 
# Defining the names of the columns referring to the items
colnames(df2)[3:37] <- paste("Blot", (1:35), sep="_") 
# Selecting only the BLOT items
blot_items <- dplyr::select(df2, Blot_1:Blot_35) 
# Looking at the first five rows of the blot dataframe
head(blot_items)

# Estimates Rasch model parameters
fit_rasch_prt <- RM(prt_items) 

# Estimating person parameters for each observed raw score. 
pparameters_prt <- eRm::person.parameter(fit_rasch_prt) 

# Estimate item parameters
item_fit_prt <- eRm::itemfit(pparameters_prt) 
print(item_fit_prt)

print(item_fit_prt, sort_by = "p")

# Print the first five rows of standardized residuals
head(item_fit_prt$st.res)

# Reshape data into long format
prt_res<-reshape2::melt(item_fit_prt$st.res)
# add new column names
colnames(prt_res) <- c("person", "item", "st.res")
# short the data, and see the top 10 rows
prt_res[order(prt_res$st.res),][1:10,]

# Person 54's response to item 8
prt_items[54,8]

# Person 54's ability score
pparameters_prt$thetapar$NAgroup1["P54"]

# Difficulty of the items
fit_rasch_prt$etapar

# Wright map
plotPImap(fit_rasch_prt, irug = TRUE, sort = TRUE) 

# Pathway map with
# 95% confidence intervals for the item parameters
plotPWmap(fit_rasch_prt, itemCI = list()) 

# Estimate Rasch model parameters for the BLOT items
fit_rasch_blot <- RM(blot_items) 
# Estimate person fit to get the person ability scores
pparameters_blot <- person.parameter(fit_rasch_blot) 

# Create a data frame of ability scores and standard errors of the estimate
combined_data <- Reduce(function(x, y) merge(x, y, all=TRUE), 
            list(
              data.frame(id = names(coef(pparameters_prt, extrapolated = TRUE)),
                         theta_prt = coef(pparameters_prt, extrapolated = TRUE),
                         row.names=NULL),
              
              data.frame(id=names(pparameters_prt$se.theta$NAgroup1),
                         se_prt= pparameters_prt$se.theta$NAgroup1, 
                         row.names=NULL),
              
              data.frame(id=names(coef(pparameters_blot, extrapolated = TRUE)),
                         theta_blot = coef(pparameters_blot, extrapolated = TRUE), 
                         row.names=NULL),
              
              data.frame(id=names(pparameters_blot$se.theta$NAgroup1),
                         se_blot = pparameters_blot$se.theta$NAgroup1, 
                         row.names=NULL)
            ))

head(combined_data[c("id","theta_blot", "theta_prt")])


# Calculate the average of the scores
(mean_prt  <-  mean(combined_data$theta_prt, na.rm = TRUE))
(mean_blot <- mean(combined_data$theta_blot, na.rm = TRUE))

# Adjust the PRTIII person measures by incorporating the mean difference
combined_data$theta_prt_adjusted <- combined_data$theta_prt + (mean_blot - mean_prt)
# Adjust the BLOT person measures by incorporating the opposite mean difference
combined_data$theta_blot_adjusted <- combined_data$theta_blot + (mean_prt - mean_blot)


# CI LOWER - BLOT scale
combined_data$blot_CI_low <- (combined_data$theta_blot+combined_data$theta_prt_adjusted)/2 - sqrt((combined_data$se_prt)^2 + (combined_data$se_blot)^2)

# CI UPPER - BLOT scale
combined_data$blot_CI_high <- (combined_data$theta_blot+combined_data$theta_prt_adjusted)/2 + sqrt((combined_data$se_prt)^2 + (combined_data$se_blot)^2)

# plot - BLOT scale
ggplot(combined_data) + 
  # Label the axes
  xlab("BLOT Estimates") + ylab("PRTIII Adjusted Estimates") +
  ggtitle("Common person linking BLOT and PRTIII") +
  # plot scatter plot
  geom_point(aes(x=theta_blot, y=theta_prt_adjusted),color="blue") +
  # add in lower CI based on a second degree polynomial
  geom_smooth(aes(x=blot_CI_low, y=blot_CI_high),color="black",
              fullrange =TRUE, se=FALSE,
              method="lm", formula = y ~ poly(x, 2)) +
  # add in upper CI based on a second degree polynomial
  geom_smooth(aes(x=blot_CI_high, y=blot_CI_low),color="black",
              fullrange =TRUE, se=FALSE,
              method="lm", formula = y ~ poly(x, 2)
  ) 


